import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.io.StringReader;
import java.sql.*;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.TreeMap;

import javax.swing.JFileChooser;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;

/* 
 * Parsing of XML requires more memory:
 * VM arguments : -Xms256m -Xmx256m
 * 
 * This program presupposes the following:
 * MySQL database : mydb
 * username : root
 * password : password
 * 
 * The table slovdb : 
 * 
 * CREATE  TABLE IF NOT EXISTS `mydb`.`slovdb` (
  `id` VARCHAR(90) NOT NULL ,
  `type` VARCHAR(45) NULL ,
  `relationships` MEDIUMTEXT NULL ,
  PRIMARY KEY (`id`) )
  ENGINE = MyISAM
 * 
 * Jar-files:
 * mysql-connector-java
 * jdom
 * 
 */

public class CountRelationsInFullHierarchy 
{
	Connection con;
	TreeMap IDVisitedCounter;
	TreeMap typeCountDuplicated;
	TreeMap uniqueIDsOfType;
		
	public CountRelationsInFullHierarchy()
	{
		con = getConnection("root", "password");
		IDVisitedCounter = new TreeMap();
		typeCountDuplicated = new TreeMap();
		uniqueIDsOfType = new TreeMap();
		
		startSearch();
		
		uniqueIDsOfType = countUniqueIDsOfType(IDVisitedCounter);
		
		writeMapSortedByValue(IDVisitedCounter, "IDVisitedCounter");
		writeMap(typeCountDuplicated, "typeCountDuplicated");
		writeMap(uniqueIDsOfType, "uniqueIDsOfType");
		
		closeConnection(con);
	}
	//this searches through the "top"-layer
	public void startSearch()
	{
		int recordCounter = 0;
		
		try
		{
			Statement sm = con.createStatement();
			String sql = "SELECT * FROM slovdb WHERE type = 4.4"; //We run through all Manifestations
			ResultSet rs = sm.executeQuery(sql);
			while(rs.next())
			{
				recordCounter++;
				String id = rs.getString(1);
				String type = rs.getString(2);
				countID(type, typeCountDuplicated);
				String relationships = rs.getString(3);
				id = type + "-" + id;
				countID(id, IDVisitedCounter);
				TreeMap visitedRecords = new TreeMap();
				int relCounter = 0;
				Iterator it = getIteratorOfElements(relationships);
				while(it.hasNext())
				{
					relCounter++;
					Element relationship = (Element) it.next();
					String relType = relationship.getAttributeValue("type");
					String href = relationship.getAttributeValue("href");
					String targetType = relationship.getAttributeValue("target_type");
					href = targetType + "-" + href;
					//counting Actors in relation to Manifestation
					if(targetType.equals("4.6"))
					{
						countID(href, IDVisitedCounter);
						countID("4.6", typeCountDuplicated);
					}					
					else if(targetType.equals("4.7"))
					{
						countID(href, IDVisitedCounter);
						countID("4.7", typeCountDuplicated);
					}
					//Manifestation 5.2.1.2.R=is the embodiment of Expression
					if(relType.equals("5.2.1.2.R") && !visitedRecords.containsKey(href))
					{
						getRecord(href);
					}
				}
				if(recordCounter % 100 == 0)
				{
					System.out.println("Recordnr." + recordCounter + " is processed");
				}
			}
			System.out.println(recordCounter + " records processed");
		}
		catch(Exception e)
		{
			System.out.println("Exception in startSearch(): " + e.getMessage());
			e.printStackTrace();
		}
	}
	
	public void getRecord(String searchForThisID)
	{
		try
		{
			Statement sm = con.createStatement();
			String searchForThisIDTruncated = searchForThisID.substring(4);
			String sql = "SELECT * FROM slovdb WHERE id = '" + searchForThisIDTruncated +  "'";
			ResultSet rs = sm.executeQuery(sql);
			while(rs.next())
			{
				String id = rs.getString(1);
				String type = rs.getString(2);
				countID(type, typeCountDuplicated);
				String relationships = rs.getString(3);
				id = type + "-" + id;
				countID(id, IDVisitedCounter);
				int relCounter = 0;
				Iterator it = getIteratorOfElements(relationships);
				while(it.hasNext())
				{
					relCounter++;
					Element relationship = (Element) it.next();
					String relType = relationship.getAttributeValue("type");
					String href = relationship.getAttributeValue("href");
					String targetType = relationship.getAttributeValue("target_type");
					href = targetType + "-" + href;
					
					//Expression 5.2.1.1.R=is a realization of Work
					if(type.equals("4.3") && relType.equals("5.2.1.1.R"))
					{
						getRecord(href);//, visitedRecords);
					}
					//Expression 5.2.2.3.R=is realized by Person
					else if(type.equals("4.3") && relType.equals("5.2.2.3.R"))
					{
						countID(href, IDVisitedCounter);
						countID("4.6", typeCountDuplicated);
					}
					//Expression 5.2.2.4.R=is realized by Corporate body
					else if(type.equals("4.3") && relType.equals("5.2.2.4.R"))
					{
						countID(href, IDVisitedCounter);
						countID("4.7", typeCountDuplicated);
					}
					//Work 5.2.2.1.R=is created by Person
					else if(type.equals("4.2") && relType.equals("5.2.2.1.R"))
					{
						countID(href, IDVisitedCounter);
						countID("4.6", typeCountDuplicated);
					}
					//Work 5.2.2.2.R=is created by Corporate body
					else if(type.equals("4.2") && relType.equals("5.2.2.2.R"))
					{
						countID(href, IDVisitedCounter);
						countID("4.7", typeCountDuplicated);
					}
					//Work 5.2.3.7.R=has as subject Concept
					else if(type.equals("4.2") && relType.equals("5.2.3.7.R"))
					{
						countID(href, IDVisitedCounter);
						countID("4.8", typeCountDuplicated);
					}
					//Work 5.2.3.6.R=has as subject Corporate body
					else if(type.equals("4.2") && relType.equals("5.2.3.6.R"))
					{
						countID(href, IDVisitedCounter);
						countID("4.7", typeCountDuplicated);
					}
					//Work 5.2.3.10.R=has as subject Place
					else if(type.equals("4.2") && relType.equals("5.2.3.10.R"))
					{
						countID(href, IDVisitedCounter);
						countID("4.11", typeCountDuplicated);
					}
					//Work 5.2.3.5.R=has as subject Person
					//this relationship was missing at the time
				}
			}
			rs.close(); //IMPORTANT TO CLOSE THIS FOR MEMORY PURPOSES!
		}
		catch(Exception e)
		{
			System.out.println("Exception in getRecord(): " + e.getMessage());
			e.printStackTrace();
		}
	}

	public TreeMap countUniqueIDsOfType(TreeMap IDMap)
	{
		TreeMap uniqueIDsOfType = new TreeMap();
		
		ArrayList IDArray = mapToArray(IDMap);
		
		for(int i = 0; i < IDArray.size(); i++)
		{
			mapping m = (mapping)IDArray.get(i);
			String id = m.getKey();
			String type = id.substring(0, 4);
			if(type.charAt(3) == '-') //all other types than 4.11 will have an unwanted '-' character
			{
				type = type.substring(0, 3);
			}
			countID(type, uniqueIDsOfType);
		}	
		
		return uniqueIDsOfType;
	}
	
	public Iterator getIteratorOfElements(String document)
	{
		Iterator it = null;
		try{
		SAXBuilder saxBuilder=new SAXBuilder();
		Reader stringReader=new StringReader(document);
		org.jdom.Document jdomDocument=saxBuilder.build(stringReader);
		Element e = jdomDocument.getRootElement();
		List recordList = e.getChildren();
		it = recordList.iterator();
		return it;
		}
		catch(Exception e)
		{System.out.println("Exception in getIteratorOfElements()");}
		
		return it;
	}
	
	public void countID(String id, TreeMap map)
	{
		if(!map.containsKey(id))
		{
			map.put(id, 1);
		}
		else
		{
			map.put(id, (Integer)map.get(id)+1);
		}
	}
	
	public Connection getConnection(String user, String password)
	{
		Connection con = null;
		try
		{
			Class.forName("com.mysql.jdbc.Driver").newInstance();
			con = DriverManager.getConnection("jdbc:mysql:///mydb", user, password);

			if(!con.isClosed())
			{
				System.out.println("Successfully connected to MySQL server using TCP/IP...");
			}
		}
		catch(Exception e)
		{
			System.out.println("Exception in getConnection: " + e.getMessage());
		}
	    return con;
	}
	
	public void closeConnection(Connection con)
	{
		try 
		{
			if(con != null)
		    {
				con.close();
		        System.out.println("Disconnected");
		    }
		}
		catch(SQLException e) {System.out.println("Exception in closeConnection: " + e.getMessage());}
	}
	
	public void writeMap(TreeMap m, String filename)
	{
		filename += ".txt";
		try{
		FileOutputStream fos = new FileOutputStream(filename);
		OutputStreamWriter out = new OutputStreamWriter(fos, "UTF-8");
		BufferedWriter bw = new BufferedWriter(out);
		bw.write(m.size() + " mappings\n");
		Set t = (Set) m.entrySet();
		Iterator setIterator = t.iterator();
		while(setIterator.hasNext())
		{
			Object o = setIterator.next();
			bw.write(o.toString()+ "\n");
		}
		bw.close();
		System.out.println("Wrote to file: " + filename);
		}
		catch(Exception e)
		{
			System.out.println("Exception in writeMap()" + e.toString());
		}
	}
	
	public void writeMapSortedByValue(TreeMap m, String filename)
	{
		filename += "-sorted";
		Set t = (Set) m.keySet();
		Iterator setIterator = t.iterator();
		ArrayList array = new ArrayList();
		while(setIterator.hasNext())
		{
			String s = (String)setIterator.next();
			mapping map = new mapping(s, (Integer)m.get(s));
			array.add(map);
		}
		writeArrayListSorted(array, filename);
	}
	
	public void writeArrayListSorted(ArrayList a, String filename)
	{	
		filename += ".txt";
		Object o[] = a.toArray();
		Arrays.sort(o);
		a = new ArrayList(Arrays.asList(o));
		try{
			FileOutputStream fos = new FileOutputStream(filename);
			OutputStreamWriter out = new OutputStreamWriter(fos, "UTF-8");
			BufferedWriter bw = new BufferedWriter(out);
			bw.write(a.size() + " mappings\n");
			//this should only be executed when array class is mapping
			//other classes cannot go through this sequence(such as triMap)
			int relSum=0;
			for(int i=0 ; i< a.size(); i++)
			{
				if(a.get(0).getClass().getSimpleName().equals("mapping"))
				{
					int numberOfRelationships = ((mapping)a.get(i)).getValue();
					relSum += numberOfRelationships;
				}
				bw.write(a.get(i).toString()+ "\n");
			}
			//casting to float to achieve float in the average value
			bw.write("Average value; Relationships:" + relSum + "/Mappings:" + a.size() + "=" + (float)relSum/a.size() + "\n");

			bw.close();
			System.out.println("Wrote to file: " + filename);
			}
			catch(Exception e)
			{
				System.out.println("Exception in writeArrayList()" + e.toString());
				e.printStackTrace();
			}
	}
	
	public ArrayList mapToArray(TreeMap m)
	{
		Set t = (Set) m.keySet();
		Iterator setIterator = t.iterator();
		ArrayList array = new ArrayList();
		while(setIterator.hasNext())
		{
			Object o = setIterator.next();
			String s = o.toString();
			//checking if the key of the map is String, because sometimes it is also an Integer
			if(o.getClass().getSimpleName().equals("String"))
			{
				mapping map = new mapping(s, (Integer)m.get(s));
				array.add(map);
			}
			else			
			{ 
				int key = Integer.parseInt(s);
				mapping map = new mapping(s, (Integer)m.get(key));
				array.add(map);
			}			
		}
		Object o[] = array.toArray();
		Arrays.sort(o);
		array = new ArrayList(Arrays.asList(o));
		return array;
	}
	
	public class mapping implements Comparable
	{
		private String key;
		private int value;
		
		public mapping(){key="";value=0;}
		public mapping(String k, int v){key = k; value = v;}
		public String getKey(){return key;}
		public int getValue(){return value;}
		public void setKey(String k){key=k;}
		public void setValue(int v){value=v;}
		public String toString(){return key + "=" + value;}
		
		public int compareTo(Object o)
		{
			try{
				mapping m = (mapping)o;
				if(value < m.getValue())
				{return 1;}
				else if (value == m.getValue())
				{return 0;}
				else if (value > m.getValue())
				{return -1;}
			}
			catch(ClassCastException cce)
			{System.out.println("Exception in compareTo():" + cce.toString());}
			return 0;
		}
	}

	public void listMap(TreeMap m)
	{
		Set t = (Set) m.entrySet();
		Iterator setIterator = t.iterator();
		while(setIterator.hasNext())
		{
			Object o = setIterator.next();
			System.out.print(o.toString() + " ");
		}
	}
	
	public static String returnTime(double milliseconds)
	{
		DecimalFormat threeDecimals = new DecimalFormat("0.000");
		double seconds = milliseconds/1000.00;
		String timeString = "";
		if(seconds > 3600)
		{
			int hours = (int)seconds/3600;
			seconds = seconds % 3600.00;
			timeString += hours + " h ";
		}
		if(seconds > 60)
		{
			int minutes = (int)seconds/60;
			seconds = seconds % 60.00;
			timeString += minutes + " min ";
		}
		timeString += threeDecimals.format(seconds) + " seconds";
		
		return timeString;
	}
	
	public static void main(String[] args)
	{
		double start = System.currentTimeMillis();
		CountRelationsInFullHierarchy c = new CountRelationsInFullHierarchy();
		double time = System.currentTimeMillis() - start;
		System.out.println("Time spent: " + returnTime(time));
	}
}
