import java.io.File;
import java.sql.*;
import java.text.DecimalFormat;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Pattern;

import javax.swing.JFileChooser;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;

/* 
 * Parsing of XML requires more memory:
 * VM arguments : -Xms256m -Xmx256m
 * 
 * This program presupposes the following:
 * MySQL database : mydb
 * username : root
 * password : password
 * 
 * The table slovdb : 
 * 
 * CREATE  TABLE IF NOT EXISTS `mydb`.`slovdb` (
  `id` VARCHAR(90) NOT NULL ,
  `type` VARCHAR(45) NULL ,
  `relationships` MEDIUMTEXT NULL ,
  PRIMARY KEY (`id`) )
ENGINE = MyISAM
 * 
 * Jar-files:
 * mysql-connector-java
 * jdom
 * 
 */

public class InsertRows 
{
	public Connection getConnection(String user, String password)
	{
		Connection con = null;
		try
		{
			Class.forName("com.mysql.jdbc.Driver").newInstance();
			con = DriverManager.getConnection("jdbc:mysql:///mydb", user, password);

			if(!con.isClosed())
			{
				System.out.println("Successfully connected to MySQL server using TCP/IP...");
			}
		}
		catch(Exception e)
		{
			System.out.println("Exception in getConnection: " + e.getMessage());
		}
	    return con;
	}
	
	public void closeConnection(Connection con)
	{
		try 
		{
			if(con != null)
		    {
				con.close();
		        System.out.println("Disconnected");
		    }
		}
		catch(SQLException e) {System.out.println("Exception in closeConnection: " + e.getMessage());}
	}
	
    public static String join(String[] s, String delimiter) {
        StringBuffer buffer = new StringBuffer();
        int max = s.length;
        for (int j = 0; j < s.length; j++)
        {
        	if (s[j] == null)
        	{
        		max = j;
        		break;
        	}
        }        
        for (int i = 0; i < max; i++)
        {
        	buffer.append(s[i]);
        	if (i != max-1)
        		buffer.append(delimiter);
        }
        return buffer.toString();
    }
	
	public InsertRows()
	{
		Connection con = getConnection("root", "password");
		int recordCounter = 0;
		
		try
		{
			JFileChooser fc = new JFileChooser();
		
			//Enable multiple file selection
			fc.setMultiSelectionEnabled(true);
			int option = fc.showOpenDialog(null);
			if(option==JFileChooser.APPROVE_OPTION)
			{
				double start = System.currentTimeMillis();
				File[] listOfFiles = fc.getSelectedFiles();
				for(int i=0;i<listOfFiles.length;i++)
				{
					File f = listOfFiles[i];
					System.out.println("Building file : " + f.toString());
					Document doc = new SAXBuilder().build(f.toString());
					Element e = doc.getRootElement();
					List recordList = e.getChildren();
					Iterator it = recordList.iterator();
					String sql = "INSERT DELAYED INTO slovdb VALUES ";
					String[] sqlAdd = new String[1000];
					Statement stmt = con.createStatement();
					int j = 0;
					while(it.hasNext())
					{
						recordCounter++;
						Element record = (Element) it.next();
						String id = record.getAttributeValue("id");
						String type = record.getAttributeValue("type");
						List childrenList = record.getChildren();
						Iterator cIt = childrenList.iterator();
						String relationshipsString = "";
						relationshipsString = "<relationships>";
						while(cIt.hasNext())
						{
							Element el = (Element) cIt.next();
							if(el.getName().equals("relationship"))
							{
								String aRelationship = "<" + el.getName() + " type=\""
								+ el.getAttributeValue("type") + "\" label=\""
								+ el.getAttributeValue("label")+ "\" target_type=\""
								+ el.getAttributeValue("target_type") + "\" target_type_label=\""
								+ el.getAttributeValue("target_type_label") + "\" href=\""
								+ el.getAttributeValue("href") + "\"/>";
								relationshipsString += aRelationship;
							}
						}
						relationshipsString += "</relationships>";
						sqlAdd[j] = "('" + id + "', '" + type + "', '" + Pattern.compile("'").matcher(relationshipsString).replaceAll("\\'") + "')";
						j++;
						if(recordCounter % 1000 == 0)
						{
							stmt.executeUpdate(sql + InsertRows.join(sqlAdd, ", \n"));
							System.out.println(recordCounter + " rows inserted.");
							sqlAdd = new String[1000];
							j = 0;
						}
					}
					if(recordCounter % 1000 != 0)
					{
						stmt.executeUpdate(sql + InsertRows.join(sqlAdd, ", \n"));
						System.out.println(recordCounter + " rows inserted.");
					}
				}
				double time = System.currentTimeMillis() - start;
				System.out.println("Time spent: " + returnTime(time));
			}
		}
		catch(Exception ex)
		{
			System.out.println("Exception in constructor: " + ex.getMessage());
		}
		finally
		{
			closeConnection(con);
		}
	}
	
	public static String returnTime(double milliseconds)
	{
		DecimalFormat threeDecimals = new DecimalFormat("0.000");
		double seconds = milliseconds/1000.00;
		String timeString = "";
		if(seconds > 3600)
		{
			int hours = (int)seconds/3600;
			seconds = seconds % 3600.00;
			timeString += hours + " h ";
		}
		if(seconds > 60)
		{
			int minutes = (int)seconds/60;
			seconds = seconds % 60.00;
			timeString += minutes + " min ";
		}
		timeString += threeDecimals.format(seconds) + " seconds";
		
		return timeString;
	}
	
	public static void main(String[] args)
	{
		InsertRows ir = new InsertRows();	
		System.exit(0);
	}
}
