<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0" xmlns:frbr="http://bibsys.no/frbrized"> 
  <xsl:output method="xml" indent="yes" encoding="UTF-8"/>
  <xsl:template match="/">
    <collection xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance'
      xsi:noNamespaceSchemaLocation='FRBRoo_Core.alt7-AEdit.xsd'>
      <xsl:call-template name="getAllManifestations"/>
    <!--  <xsl:call-template name="getAllExpressions"/>
      <xsl:call-template name="getAllWorks"/>
      <xsl:call-template name="getAllActors"/>
      <xsl:call-template name="getAllSubjects"/> -->
    </collection>
  </xsl:template>
  
  <xsl:template name="recordType">
    <id>
      <xsl:attribute name="name_space">
        <xsl:text>http://bibsys.no/frbrized</xsl:text>
      </xsl:attribute>
    </id>
    <!-- potential fields for maintenance
    <status/>
    <created/>
    -->
  </xsl:template>
  
  <xsl:template name="idrefType">
    <idref>
      <xsl:attribute name="name_space">
        <xsl:text>http://bibsys.no/frbrized</xsl:text>
      </xsl:attribute>
      <xsl:value-of select="@href"/>
    </idref>
  </xsl:template>
  
  <xsl:template name="getAllManifestations">
    <xsl:for-each select="//frbr:record[@type = '4.4']">
      <record>
        <xsl:call-template name="recordType"/>
        <manifestation>
          <xsl:call-template name="entityType"/>
          <!-- potential fields
          <is_part_of/> 
          <has_part/>
          <publication/>
          -->
          
          <!--5.2.2.5.R=is produced by 4.6=Person -->
          <!--5.2.2.5.R=is produced by 4.7=Corporate body -->
          <xsl:for-each select="frbr:relationship[@type='5.2.2.5.R']">
            <production>
              <xsl:call-template name="getEvent">
                <xsl:with-param name="actor_href" select="@href"/>
              </xsl:call-template>
            </production>
          </xsl:for-each>

          <xsl:for-each select="frbr:relationship[@type='5.2.1.2.R'][@target_type='4.3']">
            <xsl:variable name="ehref" select="@href"/>
            <carries>
              <xsl:call-template name="getExpression">
                <xsl:with-param name="ehref" select="$ehref"/>
              </xsl:call-template>
            </carries>
          </xsl:for-each>
        </manifestation>
      </record>
    </xsl:for-each>
  </xsl:template>
  
  <xsl:template name="getAllExpressions">
    <xsl:for-each select="//frbr:record[@type='4.3']">
      <xsl:variable name="ehref" select="@id"/>
      <record>
        <xsl:call-template name="recordType"/>
        <xsl:call-template name="getExpression">
          <xsl:with-param name="ehref" select="$ehref"/>
        </xsl:call-template>
      </record>
    </xsl:for-each>
  </xsl:template>
  
  <!--getExpression, retrieves an Expression based on the parameter ehref-->
  <xsl:template name="getExpression">
    <xsl:param name="ehref"/>
    <xsl:for-each select="//frbr:record[@type='4.3'][@id = $ehref]">
      <expression>
        <xsl:call-template name="entityType"/>
        <!-- potential fields
        <is_part_of/>
        <has_part/>
        </> -->
        
        <!--5.2.2.3.R=is realized by 4.6=Person -->
        <xsl:for-each select="frbr:relationship[@type='5.2.2.3.R']">
          <realisation>
            <xsl:call-template name="getEvent">
              <xsl:with-param name="actor_href" select="@href"/>
            </xsl:call-template>
          </realisation>
        </xsl:for-each>
        
        <!--5.2.2.4.R=is realized by 4.7=Corporate Body -->
        <xsl:for-each select="frbr:relationship[@type='5.2.2.4.R']">
          <realisation>
            <xsl:call-template name="getEvent">
              <xsl:with-param name="actor_href" select="@href"/>
            </xsl:call-template>
          </realisation>
        </xsl:for-each>
        
        <realises>
          <xsl:for-each select="frbr:relationship[@type='5.2.1.1.R'][@target_type='4.2']">
            <xsl:variable name="whref" select="@href"/>   
            <xsl:call-template name="getWork">
              <xsl:with-param name="whref" select="$whref"/>
            </xsl:call-template>
          </xsl:for-each>      
        </realises>
      </expression>
    </xsl:for-each>
  </xsl:template>
  
  <xsl:template name="getAllWorks">
     <xsl:for-each select="//frbr:record[@type='4.2']">
       <xsl:variable name="whref" select="@id"/>
       <record>
         <xsl:call-template name="recordType"/>
         <xsl:call-template name="getWork">
           <xsl:with-param name="whref" select="$whref"/>
         </xsl:call-template>
       </record>
     </xsl:for-each>
  </xsl:template>
  
  <!--getWork, retrieves a Work based on the parameter whref -->
  <xsl:template name="getWork">
    <xsl:param name="whref"/>
    <xsl:for-each select="//frbr:record[@type = '4.2'][@id = $whref]">
      <work>
        <xsl:call-template name="entityType"/>
        
        <!--5.2.2.1.R=is created by 4.6=Person -->
        <xsl:for-each select="frbr:relationship[@type='5.2.2.1.R']">
          <creation>
            <xsl:call-template name="getEvent">
              <xsl:with-param name="actor_href" select="@href"/>
            </xsl:call-template>
          </creation>
        </xsl:for-each>
        
        <!--5.2.2.2.R=is created by 4.7=Corporate Body -->
        <xsl:for-each select="frbr:relationship[@type='5.2.2.2.R']">
          <creation>
            <xsl:call-template name="getEvent">
              <xsl:with-param name="actor_href" select="@href"/>
            </xsl:call-template>
          </creation>
        </xsl:for-each>
        
        <!--5.3.1.8.F=is part of 4.2=Work -->    
        <!--Has to use idrefType due to looping problem with inverse relationship-->
        <xsl:for-each select="frbr:relationship[@type='5.3.1.8.F']"> <!--[@target_type='4.2']">-->
          <is_part_of>
            <xsl:call-template name="idrefType"/>
          </is_part_of>
        </xsl:for-each>
        
        <!--5.3.1.8.R=has part 4.2=Work -->
        <!--Has to use idrefType due to looping problem with inverse relationship-->
        <xsl:for-each select="frbr:relationship[@type='5.3.1.8.R']"> <!--[@target_type='4.2']">-->
          <has_part>
            <xsl:call-template name="idrefType"/>
          </has_part>
        </xsl:for-each>
      
        <!--5.2.3.7.R=has as subject 4.8=Concept-->             
        <xsl:for-each select="frbr:relationship[@type='5.2.3.7.R']">          
          <has_subject>
            <xsl:call-template name="getSubject">
              <xsl:with-param name="subject_href" select="@href"/>
            </xsl:call-template>
          </has_subject>
        </xsl:for-each>
         
        <!--5.2.3.10.R=has as subject 4.11=Place-->
        <xsl:for-each select="frbr:relationship[@type='5.2.3.10.R']">
	  <has_subject>
	    <xsl:call-template name="getSubject">
	      <xsl:with-param name="subject_href" select="@href"/>
	    </xsl:call-template>
	  </has_subject>
        </xsl:for-each>
        
        <!--5.2.3.6.R=has as subject 4.7=Corporate body-->
        <xsl:for-each select="frbr:relationship[@type='5.2.3.6.R']">         
          <has_subject>
            <xsl:call-template name="getActor">
              <xsl:with-param name="actor_href" select="@href"/>
            </xsl:call-template>
          </has_subject>
        </xsl:for-each>
        
        <!--Work 5.2.3.1.F=is subject of 4.2=Work is expressed in general <relation> element-->
        <!--5.2.3.1.R=has as subject 4.2=Work -->
        <xsl:for-each select="frbr:relationship[@type='5.2.3.1.R']">
          <has_subject>
            <xsl:call-template name="getWork">
              <xsl:with-param name="whref" select="@href"/>
            </xsl:call-template>
          </has_subject>
        </xsl:for-each>
        
        <!--5.3.1.6.F=supplements 4.2=Work -->
        <!--Has to use idrefType due to looping problem with inverse relationship-->
        <xsl:for-each select="frbr:relationship[@type='5.3.1.6.F']">
          <is_supplement_of>
            <xsl:call-template name="idrefType"/>
          </is_supplement_of>
        </xsl:for-each>
        
        <!--5.3.1.6.R=has a supplement 4.2=Work -->
        <!--Has to use idrefType due to looping problem with inverse relationship-->
        <xsl:for-each select="frbr:relationship[@type='5.3.1.6.R']">
          <has_supplement>
            <xsl:call-template name="idrefType"/>
          </has_supplement>
        </xsl:for-each>   
      </work>
    </xsl:for-each>
  </xsl:template>
    
  <!-- getAllActors; retrieves both Persons and Corporate Bodies -->
  <xsl:template name="getAllActors">
    <xsl:for-each select="//frbr:record[@type='4.6']">
      <record>
        <xsl:call-template name="recordType"/>
        <xsl:call-template name="getActor">
          <xsl:with-param name="actor_href" select="@id"/>
        </xsl:call-template>
      </record>
    </xsl:for-each>
    <xsl:for-each select="//frbr:record[@type='4.7']">      
      <record>
        <xsl:call-template name="recordType"/>
        <xsl:call-template name="getActor">
          <xsl:with-param name="actor_href" select="@id"/>
        </xsl:call-template>
      </record>
    </xsl:for-each>
  </xsl:template>
    
  <xsl:template name="getActor">
    <xsl:param name="actor_href"/>
    <xsl:for-each select="//frbr:record[@id = $actor_href]">
      <actor>
        <xsl:call-template name="entityType"/>
        <!-- potential fields
        <name></name>
        <date></date>
        <role></role> -->
        
        <!--Person 5.2.3.5.F=is subject of 4.2=Work -->
        <!--CorpBd 5.2.3.6.F=is subject of 4.2=Work -->
        <!--Both of the above relationships are contained
            in the general <relation> element.
            These are handled in 'entityType'
        -->

        <!--Person 5.2.2.1.F=has created 4.2=Work -->        
        <xsl:for-each select="frbr:relationship[@type='5.2.2.1.F']">
          <has_created>
            <xsl:call-template name="idrefType"/>
          </has_created>
        </xsl:for-each>

        <!--CorpBd 5.2.2.2.F=has created 4.2=Work -->
        <xsl:for-each select="frbr:relationship[@type='5.2.2.2.F']">
          <has_created>
	    <xsl:call-template name="idrefType"/>
          </has_created>
        </xsl:for-each>
        
        <!--Person 5.2.2.3.F=has realized 4.3=Expression -->
        <xsl:for-each select="frbr:relationship[@type='5.2.2.3.F']">
          <has_realised>
            <xsl:call-template name="idrefType"/>
          </has_realised>
        </xsl:for-each>
          
        <!--CorpBd 5.2.2.4.F=has realized 4.3=Expression -->
        <xsl:for-each select="frbr:relationship[@type='5.2.2.4.F']">
          <has_realised>
	    <xsl:call-template name="idrefType"/>
          </has_realised>
        </xsl:for-each>
        
        <!--Person 5.2.2.5.F=has produced 4.4=Manifestation -->
        <!--CorpBd 5.2.2.5.F=has produced 4.4=Manifestation -->
        <xsl:for-each select="frbr:relationship[@type='5.2.2.5.F']">
          <has_produced>
	    <xsl:call-template name="idrefType"/>
          </has_produced>
        </xsl:for-each>
      </actor>
    </xsl:for-each>
  </xsl:template>
      
  <xsl:template name="getAllSubjects">
    <xsl:for-each select="//frbr:record[@type='4.8']">
      <record>
        <xsl:call-template name="recordType"/>
        <xsl:call-template name="getSubject">
          <xsl:with-param name="subject_href" select="@id"/>
        </xsl:call-template>
      </record>
    </xsl:for-each>
    <xsl:for-each select="//frbr:record[@type='4.11']">     
      <record>
        <xsl:call-template name="recordType"/>
        <xsl:call-template name="getSubject">
          <xsl:with-param name="subject_href" select="@id"/>
        </xsl:call-template>
      </record>
    </xsl:for-each>
  </xsl:template>
  
  <xsl:template name="getEvent">
    <xsl:param name="actor_href"/>
    <xsl:call-template name="entityType"/>
    <responsible_part>
      <xsl:call-template name="getActor">
        <xsl:with-param name="actor_href" select="$actor_href"/>
      </xsl:call-template>
    </responsible_part>
  </xsl:template>
  
  <xsl:template name="getSubject">
    <xsl:param name="subject_href"/>
    <xsl:for-each select="//frbr:record[@id = $subject_href]">
      <subject>
        <xsl:call-template name="entityType"/>
      </subject>
    </xsl:for-each>
  </xsl:template>
  
  <!--EntityType, presupposes context node frbr:record.
      Handles common fields such as id, controlfield, datafield and relation(general relation element)-->
  <xsl:template name="entityType">
    <id>
      <xsl:attribute name="name_space">
        <xsl:text>http://bibsys.no/frbrized</xsl:text>
      </xsl:attribute>
      <xsl:value-of select="@id"/>
    </id>
    <xsl:for-each select="controlfield">
      <controlfield>
        <xsl:attribute name="tag">
          <xsl:value-of select="@tag"/>
        </xsl:attribute>
        <xsl:value-of select="."/>
      </controlfield>
    </xsl:for-each>
    <xsl:call-template name="datafield"/>
    <xsl:call-template name="getAllRelationships"/>
  </xsl:template>
  
  <!-- getAllRelationships retrieves relationships except those in the list
       below and puts them in the general <relation>
       Presupposes context node frbr:record
  -->
  <xsl:template name="getAllRelationships">
    <xsl:for-each select="frbr:relationship">
      <xsl:choose>
        <xsl:when test="@type='5.2.3.7.R' or @type='5.2.1.2.R' or @type='5.2.1.2.F' or @type='5.2.2.1.R' or @type='5.2.2.1.F' or @type='5.2.1.1.F' or @type='5.2.1.1.R' or @type='5.2.2.3.R' or @type='5.2.2.3.F' or @type='5.3.1.6.F' or @type='5.3.1.6.R' or @type='5.2.2.5.R' or @type='5.2.2.5.F' or @type='5.3.1.8.F' or @type='5.3.1.8.R' or @type='5.2.2.2.F' or @type='5.2.2.2.R' or @type='5.2.3.10.R' or @type='5.2.3.6.R' or @type='5.2.3.1.R' or @type='5.2.2.4.R' or @type='5.2.2.4.F' or @type='5.2.2.5.R' or @type='5.2.2.5.F'">
          <!-- all these fields are expressed using language-based tags
               and therefore will not be placed in the general relation-tag-->
        </xsl:when>
        <xsl:otherwise>
          <relation>
            <xsl:attribute name="type">
              <xsl:value-of select="@label"/>
            </xsl:attribute>
           <!-- <xsl:attribute name="label">
              <xsl:value-of select="@label"/>
            </xsl:attribute> -->
            <xsl:attribute name="idref">
              <xsl:value-of select="@href"/>
            </xsl:attribute>
          </relation>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
  </xsl:template>
  
  <!--Datafield; presupposes context node frbr:record -->
  <xsl:template name="datafield"> 
    <xsl:for-each select="./datafield">
      <datafield>
        <xsl:attribute name="tag">
          <xsl:value-of select="@tag"/>
        </xsl:attribute>
        <xsl:call-template name="subfield"/>
      </datafield>
    </xsl:for-each>
  </xsl:template>
  
  <!--Subfield; presupposes context node datafield -->
  <xsl:template name="subfield">
    <xsl:for-each select = "./subfield">
      <subfield>
        <xsl:attribute name="code">
          <xsl:value-of select="@code"/>
        </xsl:attribute>
        <xsl:if test="@type">
          <xsl:attribute name="type">
            <xsl:value-of select="@type"/>
          </xsl:attribute>
        </xsl:if>
        <xsl:value-of select="."/>
      </subfield>
    </xsl:for-each>
  </xsl:template>
  
  <!--will possibly be abandoned -->
    <xsl:template name="getAllItems">
      <xsl:for-each select="//frbr:record[@type='4.5']">
        <record>
          <xsl:call-template name="recordType"/>
          <xsl:call-template name="getItem">
            <xsl:with-param name="ihref" select="@id"/>
          </xsl:call-template>
        </record>
      </xsl:for-each>
    </xsl:template>
    
    <!--getItem, retrieves an Item based on the parameter ihref-->
    <xsl:template name="getItem">
      <xsl:param name="ihref"/>
      <xsl:for-each select="//frbr:record[@type='4.5'][@id = $ihref]">
        <item>
          <xsl:call-template name="entityType"/>
        </item>
      </xsl:for-each>
    </xsl:template>
  
</xsl:stylesheet>


