import org.jdom.*;
import org.jdom.input.SAXBuilder;

import java.text.DecimalFormat;
import java.util.*;
import java.io.*;
import javax.swing.*;

/* 
 * Parsing of XML requires more memory:
 * VM arguments : -Xms256m -Xmx256m
 */

public class getTypes
{
	TreeMap typeMap;
	TreeMap typeCountMap;
	TreeMap relMap;
	TreeMap relCountMap;
	TreeMap targetMap;
	TreeMap typeRelMap;
	TreeMap workIsCreatedByPersonMap;
	TreeMap workIsRealizedThroughExpressionMap;
	TreeMap workHasAsSubjectConceptMap;
	TreeMap conceptIsSubjectOfWorkMap;
	TreeMap expressionIsEmbodiedInManifestationMap;
	TreeMap manifestationIsTheEmbodimentOfExpressionMap;
	TreeMap personHasCreatedWorkMap;
	ArrayList combinations;
	TreeMap elementMap;
	Namespace ns = Namespace.getNamespace("http://bibsys.no/frbrized");
	int totalRecs = 0;

	public getTypes()
	{
		typeMap = new TreeMap();	//potential types
		typeCountMap = new TreeMap();
		relMap = new TreeMap(); //potential relationships		
		relCountMap = new TreeMap(); //to count relationships and their number of occurrences
		targetMap = new TreeMap(); //relationships and their potential targets
		typeRelMap = new TreeMap(); //types and potential relationships they contain
		workIsCreatedByPersonMap = new TreeMap(); //Work and number of relationships to Person (as Author)
		workIsRealizedThroughExpressionMap = new TreeMap();//work and its expressions
		workHasAsSubjectConceptMap = new TreeMap(); //work and number of concepts which it has as subject
		conceptIsSubjectOfWorkMap = new TreeMap(); //concept and the number of works it is subject to
		expressionIsEmbodiedInManifestationMap = new TreeMap();//expression and its manifestations
		manifestationIsTheEmbodimentOfExpressionMap = new TreeMap();//manifestation and its expressions
		personHasCreatedWorkMap = new TreeMap();//person and the works she has created
		combinations = new ArrayList();
		elementMap = new TreeMap(); //all potential elements
		
		double start = 0;
		try
		{
			JFileChooser fc = new JFileChooser();
		
			//Enable multiple file selection
			fc.setMultiSelectionEnabled(true);
			int option = fc.showOpenDialog(null);
			if(option==JFileChooser.APPROVE_OPTION)
			{
				start = System.currentTimeMillis();
				File[] listOfFiles = fc.getSelectedFiles();
				for(int i=0;i<listOfFiles.length;i++)
				{
					File f = listOfFiles[i];
					readFile(f.toString());
					
				}
			}
		}catch(Exception ex)
		{System.out.println("Exception in constructor");}
		
		writeMap(typeMap, "typeMap");
		writeMap(relMap, "relMap");
		writeMapSortedByValue(typeCountMap, "typeCountMap");
		writeMap(relCountMap, "relCountMap");
		writeMapSortedByValue(relCountMap, "relCountMap");
		writeMapSortedByValue(workIsRealizedThroughExpressionMap, "workIsRealizedThroughExpressionMap");
		calculateDistribution(workIsRealizedThroughExpressionMap, "workIsRealizedThroughExpressionMap");
		writeMapSortedByValue(expressionIsEmbodiedInManifestationMap, "expressionIsEmbodiedInManifestationMap");
		calculateDistribution(expressionIsEmbodiedInManifestationMap, "expressionIsEmbodiedInManifestationMap");
		writeMapSortedByValue(workIsCreatedByPersonMap, "workIsCreatedByPersonMap");
		calculateDistribution(workIsCreatedByPersonMap, "workIsCreatedByPersonMap");
		writeMapSortedByValue(workHasAsSubjectConceptMap, "workHasAsSubjectConceptMap");
		calculateDistribution(workHasAsSubjectConceptMap, "workHasAsSubjectConceptMap");
		writeMapSortedByValue(conceptIsSubjectOfWorkMap, "conceptIsSubjectOfWorkMap");
		calculateDistribution(conceptIsSubjectOfWorkMap, "conceptIsSubjectOfWorkMap");
		writeMapSortedByValue(personHasCreatedWorkMap, "personHasCreatedWorkMap");
		calculateDistribution(personHasCreatedWorkMap, "personHasCreatedWorkMap");
		writeArrayListSorted(combinations, "combinations");
		writeMap(elementMap, "elementMap");
		synthesizeMaps(typeMap, relMap, targetMap, typeRelMap, combinations, "Types and Relationships Synthesized");
		
		System.out.println("Total number of records:" + totalRecs);
		double time = System.currentTimeMillis() - start;
		System.out.println("Time spent: " + returnTime(time));
	}
		
	public void readFile(String file)
	{
		try{
		System.out.println("Building file : " + file);
		Document doc = new SAXBuilder().build(file);
		//Document doc = new SAXBuilder().build("testrecords.frbrized.xml");
		Element e = doc.getRootElement();
		elementMap.put(e.getName(), "");
		List recordList = e.getChildren();
		Iterator it = recordList.iterator();
		String workID = "";
		String expID = "";
		String manID = "";
		String personID = "";
		String conceptID = "";
		int numRecs = 0;
		//iterating through frbr:records
		while(it.hasNext())
		{
			Element record = (Element) it.next();
			elementMap.put(record.getName(), "");
			//storing all potential types
			String type = record.getAttribute("type").getValue();
			String label = record.getAttribute("label").getValue();
			typeMap.put(type, label);
			
			//storing types and their occurrence frequency
			if(!typeCountMap.containsKey(type))
			{
				typeCountMap.put(type, 1);
			}
			else
			{
				typeCountMap.put(type, (Integer)typeCountMap.get(type)+1);
			}

			//preparing type relationship map to find
			//potential relationships a type has later
			if(!(typeRelMap.containsKey(type)))
			{
				ArrayList al = new ArrayList();
				typeRelMap.put(type, al);
			}
			
			//preparing work relation maps to find
			//number of relationships they have later
			if(type.equals("4.2"))
			{
				workID = record.getAttribute("id").getValue();
				if(!workIsRealizedThroughExpressionMap.containsKey(workID))
				{	workIsRealizedThroughExpressionMap.put(workID, 0);}
				if(!workHasAsSubjectConceptMap.containsKey(workID))
				{	workHasAsSubjectConceptMap.put(workID, 0);}
				if(!workIsCreatedByPersonMap.containsKey(workID))
				{	workIsCreatedByPersonMap.put(workID, 0);}
			}
						
			//preparing expression relation map to find
			//number of relationships they have later
			if(type.equals("4.3"))
			{
				expID = record.getAttribute("id").getValue();
				if(!expressionIsEmbodiedInManifestationMap.containsKey(expID))
				{	expressionIsEmbodiedInManifestationMap.put(expID, 0);}
			}
			
			if(type.equals("4.6"))
			{
				personID = record.getAttribute("id").getValue();
				if(!personHasCreatedWorkMap.containsKey(personID))
				{	personHasCreatedWorkMap.put(personID, 0);}
			}
			
			if(type.equals("4.8"))
			{
				conceptID = record.getAttribute("id").getValue();
				if(!conceptIsSubjectOfWorkMap.containsKey(conceptID))
				{	conceptIsSubjectOfWorkMap.put(conceptID, 0);}
			}
			numRecs++;
			
			List recordChildrenList = record.getChildren();
			Iterator rclIterator = recordChildrenList.iterator();
			while(rclIterator.hasNext())
			{
				Element el = (Element) rclIterator.next();
				elementMap.put(el.getName(), "");
				
				List grandchildren = el.getChildren();
				Iterator gcIterator = grandchildren.iterator();
				while(gcIterator.hasNext())
				{
					Element gcEl = (Element) gcIterator.next();
					elementMap.put(gcEl.getName(), "");
				}
				
				if(el.getName().equals("relationship"))
				{
					//storing potential relationship types
					String relType = el.getAttributeValue("type");
					String relLabel = el.getAttributeValue("label");
					relMap.put(relType, relLabel);
					
					//storing relationships and number of occurrences to determine the most frequently used
					if(!relCountMap.containsKey(relType))
					{
						relCountMap.put(relType, 1);
					}
					else
					{
						relCountMap.put(relType, (Integer)relCountMap.get(relType)+1);
					}
					
					//preparing Map for storing of possible targets
					if(!targetMap.containsKey(relType))
					{
						ArrayList arl = new ArrayList();
						targetMap.put(relType, arl);
					}
					
					//storing possible targets of relationships
					String targetType = el.getAttributeValue("target_type");
					ArrayList a = (ArrayList) targetMap.get(relType);
					if(!a.contains(targetType))
					{
						a.add(targetType);
						targetMap.put(relType, a);
					}
					
					//storing unique combinations of types, relationships and targets
					//and their number of occurrences
					triMap tri = new triMap(type, relType, targetType);
					boolean found = false;
					int i = 0;
					if(combinations.size()==0)
					{combinations.add(tri);}
					while(!found & i<combinations.size())
					{
						triMap t = (triMap)combinations.get(i);
						if (t.equalsTo(tri))
						{
							found = true;
							((triMap)combinations.get(i)).increment();
						}
						i++;
					}
					if(!found){combinations.add(tri);}
					
					//adding potential relationships that a type can have
					ArrayList arr = (ArrayList) typeRelMap.get(type);
					if(!arr.contains(relType))
					{
						arr.add(relType);
						typeRelMap.put(type, arr);
					}
					
					//if the record is 4.2="Work"
					if(type.equals("4.2"))
					{	
						//incrementing number of "5.2.1.1.F=is realized through" to Expressions
						if(relType.equals("5.2.1.1.F"))
						{
							workIsRealizedThroughExpressionMap.put(workID, (Integer)workIsRealizedThroughExpressionMap.get(workID)+1);		
						}
						//incrementing number of "5.2.2.1.R=is created by" to Person
						if(relType.equals("5.2.2.1.R"))
						{
							workIsCreatedByPersonMap.put(workID, (Integer)workIsCreatedByPersonMap.get(workID)+1);
						}
						//incrementing number of "5.2.3.7.R=has as subject" to Concept 
						if(relType.equals("5.2.3.7.R"))
						{
							String concepID = el.getAttributeValue("href");
							workHasAsSubjectConceptMap.put(workID, (Integer)workHasAsSubjectConceptMap.get(workID)+1);
							
							//incrementing number of relations a Concept record has to a Work (this would be the 5.2.3.7.F-relation) 
							if(conceptIsSubjectOfWorkMap.containsKey(concepID))
							{
								conceptIsSubjectOfWorkMap.put(concepID, (Integer)conceptIsSubjectOfWorkMap.get(concepID)+1);
							}
							else
							{
								conceptIsSubjectOfWorkMap.put(concepID, 0);
							}
						}
					}
					
					//if the record is 4.3="Expression"
					//incrementing number of "5.2.1.2.F=is embodied in" to Manifestations
					if(type.equals("4.3") && relType.equals("5.2.1.2.F"))
					{
						expressionIsEmbodiedInManifestationMap.put(expID, (Integer)expressionIsEmbodiedInManifestationMap.get(expID)+1);
					}
					
					//incrementing number of relations from Person to Work
					//if the relationship is "5.2.2.1.F=has created"
					if(type.equals("4.6") && relType.equals("5.2.2.1.F"))
					{
						personHasCreatedWorkMap.put(personID, (Integer)personHasCreatedWorkMap.get(personID)+1);
					}
				}
			}
		}
		System.out.println(numRecs + " records processed in file : " + file);
		totalRecs += numRecs;
		doc.removeContent();
		e.removeContent();
		recordList.clear();
		}
		catch(Exception e)
		{
			System.out.println("Exception in readFile():");
			e.printStackTrace();
		}
	}
	
	public void listMap(TreeMap m)
	{
		Set t = (Set) m.entrySet();
		Iterator setIterator = t.iterator();
		while(setIterator.hasNext())
		{
			Object o = setIterator.next();
			System.out.println(o.toString());
		}
	}
	
	public String returnMapToString(TreeMap m)
	{
		String listedMap = "";
		Set t = (Set) m.entrySet();
		Iterator setIterator = t.iterator();
		while(setIterator.hasNext())
		{
			Object o = setIterator.next();
			listedMap += o.toString() + "\n";
		}
		return listedMap;
	}
	
	public void writeMap(TreeMap m, String filename)
	{
		filename += ".txt";
		try{
		FileOutputStream fos = new FileOutputStream(filename);
		OutputStreamWriter out = new OutputStreamWriter(fos, "UTF-8");
		BufferedWriter bw = new BufferedWriter(out);
		bw.write(m.size() + " mappings\n");
		Set t = (Set) m.entrySet();
		Iterator setIterator = t.iterator();
		while(setIterator.hasNext())
		{
			Object o = setIterator.next();
			bw.write(o.toString()+ "\n");
		}
		bw.close();
		System.out.println("Wrote to file: " + filename);
		}
		catch(Exception e)
		{
			System.out.println("Exception in writeMap()" + e.toString());
		}
	}
	
	public void listArrayList(ArrayList a)
	{
			for(int i=0; i<a.size(); i++)
			{
				System.out.println(a.get(i).toString());
			}
	}
	
	public void writeArrayListSorted(ArrayList a, String filename)
	{	
		filename += ".txt";
		Object o[] = a.toArray();
		Arrays.sort(o);
		a = new ArrayList(Arrays.asList(o));
		try{
			FileOutputStream fos = new FileOutputStream(filename);
			OutputStreamWriter out = new OutputStreamWriter(fos, "UTF-8");
			BufferedWriter bw = new BufferedWriter(out);
			bw.write(a.size() + " mappings\n");
			//this should only be executed when array class is mapping
			//other classes cannot go through this sequence(such as triMap)
			int relSum=0;
			for(int i=0 ; i< a.size(); i++)
			{
				if(a.get(0).getClass().getSimpleName().equals("mapping"))
				{
					int numberOfRelationships = ((mapping)a.get(i)).getValue();
					relSum += numberOfRelationships;
				}
				bw.write(a.get(i).toString()+ "\n");
			}
			//casting to float to achieve float in the average value
			bw.write("Average value; Relationships:" + relSum + "/Mappings:" + a.size() + "=" + (float)relSum/a.size() + "\n");

			bw.close();
			System.out.println("Wrote to file: " + filename);
			}
			catch(Exception e)
			{
				System.out.println("Exception in writeArrayList()" + e.toString());
				e.printStackTrace();
			}
	}
	
	public void calculateDistribution(TreeMap m, String filename)
	{
		filename +="-distribution.txt";
		try{
			FileOutputStream fos = new FileOutputStream(filename);
			OutputStreamWriter out = new OutputStreamWriter(fos, "UTF-8");
			BufferedWriter bw = new BufferedWriter(out);
			TreeMap relationFrequencyMap = new TreeMap();
			ArrayList a = mapToArray(m);
			int relSum = 0; //sum stores the accumulated number of relationships
			//calculating how many elements have 1 relationship, 2 relationships,... N relationships...
			for(int i=0; i<a.size(); i++)
			{	
				int numberOfRelationships = ((mapping)a.get(i)).getValue();
				if(relationFrequencyMap.containsKey(numberOfRelationships))
				{
					relationFrequencyMap.put(numberOfRelationships, (Integer)relationFrequencyMap.get(numberOfRelationships)+1);
				}
				else
				{
					relationFrequencyMap.put(numberOfRelationships, 1);
				}
				relSum += numberOfRelationships;
			}
			String nonDup = calculateNonDuplicated(relationFrequencyMap) + "";
			bw.write("Non-duplicated records if referencing is used: " + nonDup + "\n"); 
			bw.write("Frequencies of relations : \n" + returnMapToString(relationFrequencyMap));
			bw.close();
			System.out.println("Wrote to file: " + filename);
		}catch(Exception e)
		{
			System.out.println("Exception in calculateDistribution():" + e.toString());
			e.printStackTrace();
		}
	}
	
	public int calculateNonDuplicated(TreeMap m)
	{
		ArrayList a = mapToArray(m);
		int nonDup = 0;
		for(int i=0; i<a.size(); i++)
		{
			int numberOfRelationships = Integer.parseInt(((mapping)a.get(i)).getKey());
			int frequency = ((mapping)a.get(i)).getValue();
			if(numberOfRelationships > 0)
			{
				nonDup += (numberOfRelationships-1)*frequency;
			}
		}
		return nonDup;
	}
	
	public ArrayList mapToArray(TreeMap m)
	{
		Set t = (Set) m.keySet();
		Iterator setIterator = t.iterator();
		ArrayList array = new ArrayList();
		while(setIterator.hasNext())
		{
			Object o = setIterator.next();
			String s = o.toString();
			//checking if the key of the map is String, because sometimes it is also an Integer
			if(o.getClass().getSimpleName().equals("String"))
			{
				mapping map = new mapping(s, (Integer)m.get(s));
				array.add(map);
			}
			else			
			{ 
				int key = Integer.parseInt(s);
				mapping map = new mapping(s, (Integer)m.get(key));
				array.add(map);
			}			
		}
		Object o[] = array.toArray();
		Arrays.sort(o);
		array = new ArrayList(Arrays.asList(o));
		return array;
	}
	
	public void writeMapSortedByValue(TreeMap m, String filename)
	{
		filename += "-sorted";
		Set t = (Set) m.keySet();
		Iterator setIterator = t.iterator();
		ArrayList array = new ArrayList();
		while(setIterator.hasNext())
		{
			String s = (String)setIterator.next();
			mapping map = new mapping(s, (Integer)m.get(s));
			array.add(map);
		}
		writeArrayListSorted(array, filename);
	}
	
	public void synthesizeMaps(TreeMap typeMap, TreeMap relMap, TreeMap targetMap, TreeMap typeRelMap, ArrayList combinations, String filename)
	{
		filename += ".txt";
		String output = "";
		Set typeRelSet = (Set)typeRelMap.keySet();
		Iterator typeRelIt = typeRelSet.iterator();
		while(typeRelIt.hasNext())
		{
			String typeID = (String)typeRelIt.next();
			String typeLabel = (String)typeMap.get(typeID);
			output += typeID + "=" + typeLabel + "-->\n"; //adding the type to the string and the name of the type
			ArrayList relList = (ArrayList)typeRelMap.get(typeID); //getting the mapped values for each key, in this case the potential relationships for each type
			for(int i=0; i<relList.size(); i++)
			{
				String relID = (String)relList.get(i);
				String relLabel = (String)relMap.get(relID);
				output += "\t\t" + relID + "=" + relLabel + "-->\n"; //adding potential relationships and their labels
				ArrayList targetList = (ArrayList)targetMap.get(relID);
				for(int j=0; j<targetList.size(); j++)
				{
					String targetID = (String)targetList.get(j);
					triMap tri = new triMap(typeID, relID, targetID);
					int count = 0;
					for(int k=0;k<combinations.size();k++)
					{	triMap t = (triMap)combinations.get(k);
						if(t.equalsTo(tri)){count = t.getCounter();}
					}
					output += "\t\t\t\t\t" + targetID + "=" + (String)typeMap.get(targetList.get(j)) + "=" + count + "\n";
				}
			}
		}
		try{
			FileOutputStream fos = new FileOutputStream(filename);
			OutputStreamWriter out = new OutputStreamWriter(fos, "UTF-8");
			BufferedWriter bw = new BufferedWriter(out);
			bw.write(output);
			bw.close();
			System.out.println("Wrote to file: " + filename);
		}catch(Exception e){System.out.println("Exception in synthesizeMaps()");}
		System.out.println(output);
	}
	
	public class mapping implements Comparable
	{
		private String key;
		private int value;
		
		public mapping(){key="";value=0;}
		public mapping(String k, int v){key = k; value = v;}
		public String getKey(){return key;}
		public int getValue(){return value;}
		public void setKey(String k){key=k;}
		public void setValue(int v){value=v;}
		public String toString(){return key + "=" + value;}
		
		public int compareTo(Object o)
		{
			try{
				mapping m = (mapping)o;
				if(value < m.getValue())
				{return 1;}
				else if (value == m.getValue())
				{return 0;}
				else if (value > m.getValue())
				{return -1;}
			}
			catch(ClassCastException cce)
			{System.out.println("Exception in compareTo():" + cce.toString());}
			return 0;
		}
	}
	//class to store a unique combination of type, relationship and target and their number of occurrences
	public class triMap implements Comparable
	{
		private String type;
		private String relationship;
		private String target;
		private int counter;
		public triMap(String t, String r, String ta)
		{	type = t; relationship = r; target = ta; counter=1;}
		public String getTypeString(){return type;}
		public String getRelationship(){return relationship;}
		public String getTarget(){return target;}
		public int getCounter(){return counter;}
		public void increment(){counter++;}
		public String toString(){return type + "-->" + relationship + "-->" + target + "=" + counter;}
		public boolean equalsTo(triMap t)
		{
			if(type.equals(t.getTypeString()) && relationship.equals(t.getRelationship()) && target.equals(t.getTarget()))
				return true;
			else
				return false;
		}
		public int compareTo(Object o)
		{
			try{
				triMap m = (triMap)o;
				if(counter < m.getCounter())
				{return 1;}
				else if (counter == m.getCounter())
				{return 0;}
				else if (counter > m.getCounter())
				{return -1;}
			}
			catch(ClassCastException cce)
			{System.out.println("Exception in compareTo():" + cce.toString());}
			return 0;
		}
	}
	
	public static String returnTime(double milliseconds)
	{
		DecimalFormat threeDecimals = new DecimalFormat("0.000");
		double seconds = milliseconds/1000.00;
		String timeString = "";
		if(seconds > 3600)
		{
			int hours = (int)seconds/3600;
			seconds = seconds % 3600.00;
			timeString += hours + " h ";
		}
		if(seconds > 60)
		{
			int minutes = (int)seconds/60;
			seconds = seconds % 60.00;
			timeString += minutes + " min ";
		}
		timeString += threeDecimals.format(seconds) + " seconds";
		
		return timeString;
	}
	
	public static void main(String args[])
	{
		
		getTypes g = new getTypes();
		System.exit(0);
	}
}
